#include "gameobject.h"
#include "graphics.h"

#include <cmath>

// Assumes radius <= 1

using namespace std;

static const double BorderWidth = 0.2;

bool GameObject::testLocation(const Vect3d & where)
{
	Vect3d rel = where - location;
	if (rel.magnitudeSquared() < (radius * radius)) {
		handleMapCollision(where);
		return true;
	}
	return false;
	
	/*
	MapBlock &b = manager.map.blockAt((int) where.x, (int) where.y);
	if (b.isSolid()) {
		double offsetX = fmod(where.x, 1);
		double offsetY = fmod(where.x, 1);
		bool hit = ((offsetX > BorderWidth) && (offsetX < (1.0 - BorderWidth)) &&
			(offsetY > BorderWidth) && (offsetY < (1.0 - BorderWidth)));
		if (hit) {
			handleMapCollision(where);
		}
		return hit;
	} else {
		return false;
	}
	*/
}

void GameObject::testBlock(int ix, int iy, double x, double y)
{
	MapBlock &b = manager.map.blockAt(ix,iy);
	if (b.isSolid()) {
		testLocation(Vect3d(x,y, location.z));
	}
}

void GameObject::clip()
{
	// Handle block to the right
	int ix = (int) location.x;
	int iy = (int) location.y;
	// right
	testBlock(ix + 1, iy, (double) (ix + 1), location.y);
	// Up
	testBlock(ix, iy + 1, location.x, (double) iy + 1);
	// down
	testBlock(ix, iy - 1, location.x, (double) iy);
	// left
	testBlock(ix - 1, iy, (double) ix, location.y);
	// diagonals
	testBlock(ix + 1, iy + 1, (double) (ix + 1), (double) (iy + 1));
	testBlock(ix - 1, iy + 1, (double) ix, (double) (iy + 1));
	testBlock(ix - 1, iy - 1, (double) ix, (double) iy);
	testBlock(ix + 1, iy - 1, (double) (ix + 1), (double) iy);
}

void GameObject::handleMapCollision(const Vect3d & where)
{
	// Work out overlap distance.
	// Back off by that distance.
	Vect3d rel = where - location;
	double overlap = radius - (rel.magnitude());
	Vect3d overlapVec = rel;
	overlapVec.normalise();
	overlapVec = overlapVec * overlap;
	// Back off
	location = location - overlapVec;
}

void GameObject::checkBounds()
{
	if (location.x < manager.getLeftBoundary()) {
		dead = true;
	}
	if (location.y > manager.getTopBoundary()) {
		dead = true;
	}
	if (location.y < manager.getBottomBoundary()) {
		dead = true;
	}
}

void GameObject::drawHurtFlash()
{
	if (hurtFlash > 0) {
		glDisable(GL_DEPTH_TEST);
		glDisable(GL_LIGHTING);
		glDisable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		glBlendFunc(GL_ONE, GL_SRC_ALPHA);
		DrawLuminousQuad(location, radius * 1.5, hurtFlash, hurtFlash, hurtFlash);
	}
}

void GameObject::updateHurtFlash()
{
	if (hurtFlash > 0) {
		hurtFlash -= 0.3;
	}
}

void Explosion::render()
{
	int frame = timer / FrameLength;
	// Textured quad with transparency.
	float leftx = location.x - radius;
	float rightx = location.x + radius;
	float bottomy = location.y - radius;
	float topy = location.y + radius;
	
	float texbottom = ((float) frame) / 8.0;
	float textop = texbottom + (1.0 / 8.0);
	
	manager.rm.explosiontexture.Bind();
	glDisable(GL_LIGHTING);
	glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glColor3f(1,1,1);
	glBegin(GL_QUADS);
	glTexCoord2f(0,textop);
	glVertex3f(leftx, bottomy, location.z);
	glTexCoord2f(1,textop);
	glVertex3f(rightx,bottomy,location.z);
	glTexCoord2f(1,texbottom);
	glVertex3f(rightx,topy,location.z);
	glTexCoord2f(0,texbottom);
	glVertex3f(leftx,topy,location.z);
	glEnd();
}

void Explosion::runLogic()
{
	timer++;
	if ((timer / FrameLength) > NumFrames)
		dead = true;
}

