#ifndef MAP_INCLUDED
#define MAP_INCLUDED

#include <vector>

// Each column on the map will have
// a colour used for background images.
// Also each cell will have its own tile (0 = none)

struct MapBlock {
	enum MapBlockType { EmptyBlock, GroundBlock, TowerBlock };
	enum MapItemType { EmptyItem, TurretItem1, TurretItem2, TurretItem3, 
		SaucerItem1, SaucerItem2, SaucerItem3, BonusItem };
	MapBlockType type;
	unsigned char tile;
	MapItemType item;
	bool operator == (const MapBlock &other) {
		return ((type == other.type) && (tile == other.tile));
	}
	MapBlock() : type(EmptyBlock), tile(0), item(EmptyItem) { }
	bool isSolid() {
		return (type == TowerBlock);
	}
	void setFromChar(char c) {
		item = EmptyItem;
		tile = 0;
		type = EmptyBlock;
		switch (c) {
			case '[': type = GroundBlock;
				break;
			case 'T': type = TowerBlock;
				break;
			case '1': type = TowerBlock;
				item = TurretItem1;
				break;
			case '2': type = TowerBlock;
				item = TurretItem2;
				break;
			case '3': type = TowerBlock;
				item = TurretItem3;
				break;
			case '4':
				item = SaucerItem1;
				break;
			case '5':
				item = SaucerItem2;
				break;
			default:
				// nothing
				break;
		}
	}
};

struct Colour {
	float r, g, b;
};

class ResourceManager;

class Map {
	public:
		static const int MapWidth = 1200;
		static const int MapHeight = 9;
	private:
		MapBlock emptyBlock;
		struct MapColumn {
				std::vector<MapBlock> blocks;
				Colour colour;
				int speed;
				MapColumn() : speed(1) {
					blocks.resize(MapHeight);
				}
		};
		std::vector<MapColumn> blocks;

	public:
		
		Map() {
			blocks.resize(MapWidth);
			emptyBlock.type = MapBlock::EmptyBlock;
		}
		
		MapBlock & blockAt(int x, int y) {
			if ((x<0) || (x>=MapWidth) || (y >=MapHeight)) {
				return emptyBlock;
			}
			return blocks[x].blocks[y];
		}
		int getSpeed(int x) { 
			if ((x<0) || (x>=MapWidth)) {
				return 1;
			}
			return blocks[x].speed;
		}
		
		void renderColumn(int x, const ResourceManager &rm);
		void init(int level);
};

#endif // MAP_INCLUDED
