// Model.cpp
#include "model.h"
#include <iostream>
#include <fstream>
#include <GL/gl.h>

void Model::load(string filename)
{
	// Create input stream object
	ifstream infile(filename.c_str());
	// Read the number of points
	int numpoints;
	infile >> numpoints;
	// Resize the vector to hold them
	points.resize(numpoints);
	// Then read the points' coordinates.
	for (int i=0; i<numpoints; i++) {
		// NOTICE strange order. it's right.
		infile >> points[i].x >> points[i].z >> points[i].y;
		points[i].y = - points[i].y;
	}
	// Same for polygons
	int numpolys;
	infile >> numpolys;
	polys.resize(numpolys); // resize vector as before
	// Read the polygons in
	for (int i=0; i<numpolys; i++) {
		// Read the vertex list
		int numvertices;
		infile >> numvertices;
		if (numvertices < 3) {
			std::cerr << "Strange, a polygon with less than three vertices " <<
				"Number " << i << std::endl;
			throw "Polygon with <3 verts";
		}
		polys[i].points.resize(numvertices);
		for (int j=0; j<numvertices; j++) {
			infile >> polys[i].points[j];
		}
		// Calculate the surface normal
		Vect3d firstpoint, secondpoint, thirdpoint;
		firstpoint = points[polys[i].points[0]];
		secondpoint = points[polys[i].points[1]];
		thirdpoint = points[polys[i].points[2]];
		polys[i].normal = (thirdpoint - firstpoint) * (secondpoint - firstpoint);
		// polys[i].normal =  (secondpoint - firstpoint) * (thirdpoint - firstpoint);
		// Normalise the normal (i.e. unit length)
		// Otherwise opengl lighting goes wrong.
		polys[i].normal.normalise();
	}
	// Work out min/max x/y
	minx = 1000;
	maxx = -1000;
	miny = 1000;
	maxy = -1000;
	for (int i=0; i<numpoints; i++) {
		Vect3d & p = points[i];
		minx = std::min(p.x, minx);
		miny = std::min(p.y, miny);
		maxx = std::max(p.x, maxx);
		maxy = std::max(p.y, maxy);
	}
	textureScale = 1.0 / max(maxx - minx, maxy - miny);
	// The offset of our Y coordinate, so it goes from whatever .. 1.0
	textureYOffset = 1.0 - ((maxy - miny) * textureScale);

	// All done.
}

void Model::draw(bool texture) const
{
	// Go through the polys
	int numpolys = polys.size();
	for (int i=0; i<numpolys; i++)
	{
		int numpoints = polys[i].points.size();
		glBegin(GL_POLYGON);
		glNormal3f(polys[i].normal.x,
			polys[i].normal.y,
			polys[i].normal.z);
		// Go backwards around the poly.
		for (int j = (numpoints - 1); j >=0; j--) {
			const Vect3d & p = points[polys[i].points[j]];
			if (texture) {
				// calc tex coords.
				Vect3d tx = p;
				tx.x -= minx;
				tx.y -= miny;
				tx.x *= textureScale;
				tx.y *= textureScale;
				tx.y += textureYOffset;
				glTexCoord2f(tx.x, 1.0 - tx.y);
			}
			glVertex3f(p.x,
				p.y,
				p.z);
		}
		glEnd();
	}
}
